/**
 * ASSourceFactory.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas;

import uk.co.badgersinfoil.metaas.dom.ASBlock;
import uk.co.badgersinfoil.metaas.dom.ASCompilationUnit;
import uk.co.badgersinfoil.metaas.impl.ASTActionScriptParser;
import uk.co.badgersinfoil.metaas.impl.ASTActionScriptProject;
import uk.co.badgersinfoil.metaas.impl.ASTActionScriptWriter;
import uk.co.badgersinfoil.metaas.impl.ASTBuilder;
import uk.co.badgersinfoil.metaas.impl.ASTStatementList;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;

// TODO: an isKeyword(String) test would be nice

/**
 * Core class providing access to metaas functionality.
 */
public class ActionScriptFactory {

	/**
	 * Creates a new CompilationUnit which defines a class with the given
	 * name.  To populate the new class, you can do something like,
	 * <pre>
	 * CompilationUnit cu = fact.newClass("MyTest");
	 * ASTClassType myclass = (ASTClassType)cu.getType();
	 * // ... add stuff to myclass  ...
	 * </pre>
	 */
	public ASCompilationUnit newClass(String qualifiedClassName) {
		return ASTBuilder.synthesizeClass(qualifiedClassName);
	}

	/**
	 * Creates a new CompilationUnit which defines an interface with the
	 * given name.  To populate the new interface, you can do something
	 * like,
	 * <pre>
	 * CompilationUnit cu = fact.newClass("MyTest");
	 * ASTClassType myiface = (ASTInterfaceType)cu.getType();
	 * // ... add stuff to myiface  ...
	 * </pre>
	 */
	public ASCompilationUnit newInterface(String qualifiedInterfaceName) {
		return ASTBuilder.synthesizeInterface(qualifiedInterfaceName);
	}

	public ActionScriptWriter newWriter() {
		return new ASTActionScriptWriter();
	}
	
	public ActionScriptParser newParser() {
		return new ASTActionScriptParser();
	}

	/**
	 * Escape the given String and place within double quotes so that it
	 * will be a valid ActionScript string literal.
	 */
	public static String str(String str) {
		StringBuffer result = new StringBuffer("\"");
		for (int i=0; i<str.length(); i++) {
			char c = str.charAt(i);
			switch (c) {
			    case '\n':
				result.append("\\n");
				break;
			    case '\t':
				result.append("\\t");
				break;
			    case '\r':
				result.append("\\r");
				break;
			    case '"':
				result.append("\\\"");
				break;
			    case '\\':
				result.append("\\\\");
				break;
			    default:
				result.append(c);
			}
		}
		result.append('"');
		return result.toString();
	}

	/**
	 * Creates a new ActionScript block statement.  Can be supplied to an
	 * {@link uk.co.badgersinfoil.metaas.dom.ASIfStatement}, for instance.
	 */
	public ASBlock newBlock() {
		LinkedListTree block = ASTBuilder.newBlock();
		return new ASTStatementList(block);
	}

	public ActionScriptProject newEmptyASProject(String outputLocation) {
		ASTActionScriptProject project = new ASTActionScriptProject(this);
		project.setOutputLocation(outputLocation);
		return project;
	}
}