/**
 * ASDoWhileStatement.java
 * 
 * Copyright (c) 2007 David Holroyd
 */

package uk.co.badgersinfoil.metaas.dom;


/**
 * Obtained from {@link StatementContainer#newDoWhile(String)}, represents
 * a <code>do { } while (condition);</code> construct.
 * 
 * <p>e.g. The following Java code,</p>
 * <pre class="eg">ASDoWhileStatement doWhile = method.newDoWhile("test()");
 *doWhile.addStmt("trace('still testing')");</pre>
 * <p>Will result in AS which looks like this,</p>
 * <pre class="eg">do {
 *	trace('still testing');
 *} while (test());</pre>
 */
public interface ASDoWhileStatement extends Statement, StatementContainer {

	/**
	 * Returns a string representation of the loop termination condition
	 * expression.
	 * 
	 * <p>e.g. given the loop,</p>
	 * <pre class="eg">do { nothing(); } while (test());</pre>
	 * <p>This method would return <code>"test()"</code>.</p>
	 */
	public String getConditionString();

	/**
	 * Sets the loop termination condition for this do-while loop.
	 * 
	 * @throws uk.co.badgersinfoil.metaas.SyntaxException if the given
	 * string is not a valid ActionScript expression.
	 */
	public void setCondition(String string);
}