/**
 * ASForEachInStatement.java
 * 
 * Copyright (c) 2007 David Holroyd
 */

package uk.co.badgersinfoil.metaas.dom;

/**
 * Obtained from {@link StatementContainer#newForEachIn(String,String)},
 * represents a <code>for each(v in a) { }</code> statement.
 * 
 * <p>e.g. The Java code</p>
 * <pre class="eg">ASForEachInStatement forEachIn = method.newForEachIn("var v", "arr");
 *forEachIn.addStmt("trace(v)");</pre>
 *
 * <p>Will create ActionScript code like,</p>
 * <pre class="eg">for each(var v in arr) {
 *	trace(v);
 *}</pre>
 */
public interface ASForEachInStatement extends Statement, StatementContainer {

	/**
	 * Returns a string representation of the loop-variable declaration
	 * for this loop.
	 * 
	 * e.g. given the loop <code>for each(v in a) { }</code>, this method
	 * will return the string <code>"v"</code>.
	 */
	public String getVarString();

	/**
	 * Returns a string representation of the expression whose value will
	 * iterated over.
	 * 
	 * e.g. given the loop <code>for each(v in a) { }</code>, this method
	 * will return the string <code>"a"</code>.
	 */
	public String getIteratedString();

	/**
	 * Specifies the loop-variable declaration for this loop.
	 * 
	 * @throws uk.co.badgersinfoil.metaas.SyntaxException if the given
	 * string is not a valid ActionScript expression.
	 */
	public void setVar(String expr);

	/**
	 * Changes the expression whose value will be iterated over by this
	 * loop.
	 * 
	 * @throws uk.co.badgersinfoil.metaas.SyntaxException if the given
	 * string is not a valid ActionScript expression.
	 */
	public void setIterated(String expr);
}