/**
 * ASForStatement.java
 * 
 * Copyright (c) 2007 David Holroyd
 */

package uk.co.badgersinfoil.metaas.dom;

/**
 * Obtained from {@link StatementContainer#newFor(String,String,String)},
 * represents a <code>for (; ; ) { }</code> statement.
 * 
 * <p>e.g. The Java code</p>
 * <pre class="eg">ASForStatement forStmt = method.newFor("var i=0", "i<10", "i++");
 *forStmt.addStmt("trace(i)");</pre>
 *
 * <p>Will create ActionScript code like,</p>
 * <pre class="eg">for (var i=0; i<10; i++) {
 *	trace(i);
 *}</pre>
 */
public interface ASForStatement extends Statement, StatementContainer {

	/**
	 * Returns a string representation of the loop initialisation
	 * expression.
	 * 
	 * e.g. given the loop <code>for (var i=0; i<10; i++)</code>, this
	 * method will return the string <code>"var i=0"</code>.
	 */
	public String getInitString();

	/**
	 * Returns a string representation of the loop termination condition
	 * expression.
	 * 
	 * e.g. given the loop <code>for (var i=0; i<10; i++)</code>, this
	 * method will return the string <code>"i<10"</code>.
	 */
	public String getConditionString();

	/**
	 * Returns a string representation of the loop update expression.
	 * 
	 * e.g. given the loop <code>for (var i=0; i<10; i++)</code>, this
	 * method will return the string <code>"i++"</code>.
	 */
	public String getUpdateString();

	/**
	 * Changes the initialisation expression for this loop.
	 * 
	 * @throws uk.co.badgersinfoil.metaas.SyntaxException if the given
	 * string is not a valid ActionScript expression.
	 */
	public void setInit(String expr);

	/**
	 * Changes the termination condition expression for this loop.
	 * 
	 * @throws uk.co.badgersinfoil.metaas.SyntaxException if the given
	 * string is not a valid ActionScript expression.
	 */
	public void setCondition(String expr);

	/**
	 * Changes the update expression for this loop.
	 * 
	 * @throws uk.co.badgersinfoil.metaas.SyntaxException if the given
	 * string is not a valid ActionScript expression.
	 */
	public void setUpdate(String expr);
}