/**
 * ASMember.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.dom;


/**
 * A member of a type; an {@link ASMethod} or {@link ASField}.
 */
public interface ASMember extends MetaTagable, Documentable {
	/**
	 * Returns the name of this field.
	 */
	public String getName();

	/**
	 * Changes the name of this ActionScript field to the given value.
	 */
	public void setName(String string);

	/**
	 * Returns the name of the return type of value this ActionScript
	 * field may contain, or null if it is untyped.
	 */
	public String getType();

	/**
	 * Defines the name of the type of object this ActionScript field may
	 * contain.  May be set to null, denoting that the value is
	 * untyped.
	 */
	public void setType(String string);

	/**
	 * Returns a value representing any protection-against-access defined
	 * for this ActionScript field.
	 */
	public Visibility getVisibility();

	/**
	 * Defines the level of protection-against-external-access for this
	 * ActionScript field.
	 */
	public void setVisibility(Visibility visibility);


	/**
	 * Returns true if this ActionScript field is static (i.e. the field
	 * definition uses the <code>static</code> keyword).
	 */
	public boolean isStatic();

	/**
	 * Defines this method to be static, or not.  If set to true, this
	 * member definition will be qualified with the <code>static</code>
	 * keyword, if false, the keyword will be absent.
	 */
	public void setStatic(boolean s);
}