/**
 * ASSwitchStatement.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.dom;

/**
 * Obtained from {@link StatementContainer#newSwitch(String)}, an
 * ASSwitchStatement allows 'case' and 'default' statements to be added to
 * to the 'switch' statement.
 * 
 * e.g.
 * <pre class="eg">
 * ASSwitchStatement switchStmt = method.newSwitch("c");
 * switchStmt.newCase("'a'").addStmt("aay()");
 * switchStmt.newCase("'b'").addStmt("bee()");
 * switchStmt.newDefault().addStmt("cee()");
 * </pre>
 * 
 * Will result in ActionScript something like,
 * <pre class="eg">
 * switch(c) {
 * 	case 'a':
 * 		aay();
 * 	case 'b':
 * 		bee();
 * 	default:
 * 		cee();
 * }
 * </pre>
 * 
 * Note that to add 'break' statements, you can use
 * <code>switchStmt.addStmt("break")</code>.
 */
public interface ASSwitchStatement extends Statement {

	/**
	 * Creates a <code>case</code>-label in this switch-statement with the given value,
	 * to which other statements can be added.
	 */
	ASSwitchCase newCase(String string);

	/**
	 * Creates a <code>default</code>-label in this switch-statement, to
	 * which other statements can be added.
	 */
	ASSwitchDefault newDefault();
}
