package uk.co.badgersinfoil.metaas.dom;

/**
 * Interface for parts of the ActionScript source code which can have API
 * documentation comments attached.
 * 
 * [Hopefully this interface will be extended to allow more sophisticated
 * JavaDoc-style comment processing.]
 */
public interface Documentable {

	/**
	 * Returns the contents of the documentation comment for this API
	 * element.
	 * Lines of text in the comment will alreeady have had leading '*'
	 * characters removed.  If there is no current documentation comment,
	 * null will be returned.
	 */
	public String getDocComment();

	/**
	 * Set the contents of the 'documentation comment' for this ActionScript
	 * API element.  If the given text has multiple lines, each line will
	 * have initial indentation and a '*' character added, so you shouldn't
	 * include these in the text yourself.  Supplying a null value removes
	 * any existing comment.
	 */
	public void setDocComment(String text);
}