/**
 * MetaTagable.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.dom;

import java.util.List;


/**
 * The common interface for API elements that may be tagged with metadata.
 * 
 * @see ASClassType
 * @see ASInterfaceType
 * @see ASField
 * @see ASMethod
 */
public interface MetaTagable {
	/**
	 * Returns the ASMetaTag with the given name.  If multiple metatags
	 * with the name are attached to this API element, only the first is
	 * returned.  If no such meta tag is found, this method returns null.
	 * 
	 * @see #getMetaTagsWithName(String)
	 */
	public ASMetaTag getFirstMetatag(String name);

	/**
	 * Returns the (possibly empty) list of ASMetaTag objects attached to
	 * this API element.
	 */
	public List getAllMetaTags();

	/**
	 * Returns a (possibly empty) list of ASMetaTag objects attached to
	 * this API element whose names match the given value.
	 */
	public List getMetaTagsWithName(String name);

	/**
	 * Creates and returns a new ASMetaTag attached to this API element.
	 */
	public ASMetaTag newMetaTag(String name);
}