/**
 * ASTASForStatement.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import uk.co.badgersinfoil.metaas.dom.ASForStatement;
import uk.co.badgersinfoil.metaas.dom.StatementContainer;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


public class ASTASForStatement extends ContainerDelegate implements ASForStatement {

	private static final int INDEX_INIT = 0;
	private static final int INDEX_CONDITION = 1;
	private static final int INDEX_UPDATE = 2;
	private static final int INDEX_STATEMENT = 3;
	private LinkedListTree ast;

	public ASTASForStatement(LinkedListTree ast) {
		this.ast = ast;
	}

	public String getInitString() {
		if (!hasInit()) {
			return null;
		}
		return ASTUtils.stringifyNode(findInit());
	}

	public String getConditionString() {
		if (!hasCondition()) {
			return null;
		}
		return ASTUtils.stringifyNode(findCondition());
	}

	public String getUpdateString() {
		if (!hasUpdate()) {
			return null;
		}
		return ASTUtils.stringifyNode(findUpdate());
	}

	private boolean hasCondition() {
		return findCondition().getChildCount() > 0;
	}

	private boolean hasInit() {
		return findInit().getChildCount() > 0;
	}

	private boolean hasUpdate() {
		return findUpdate().getChildCount() > 0;
	}

	private LinkedListTree getChild(int index) {
		return (LinkedListTree)ast.getChild(index);
	}

	protected StatementContainer getStatementContainer() {
		return new ASTStatementList(getChild(INDEX_STATEMENT));
	}

	private LinkedListTree findInit() {
		return getChild(INDEX_INIT);
	}

	private LinkedListTree findCondition() {
		return getChild(INDEX_CONDITION);
	}

	private LinkedListTree findUpdate() {
		return getChild(INDEX_UPDATE);
	}

	public void setCondition(String expr) {
		if (expr == null) {
			deleteAnyChild(findCondition());
		} else {
			LinkedListTree cond = AS3FragmentParser.parseForCond(expr);
			ast.setChildWithTokens(INDEX_CONDITION, cond);
		}
	}

	public void setInit(String expr) {
		if (expr == null) {
			deleteAnyChild(findInit());
		} else {
			LinkedListTree init = AS3FragmentParser.parseForInit(expr);
			ast.setChildWithTokens(INDEX_INIT, init);
		}
	}

	public void setUpdate(String expr) {
		if (expr == null) {
			deleteAnyChild(findUpdate());
		} else {
			LinkedListTree update = AS3FragmentParser.parseForIter(expr);
			ast.setChildWithTokens(INDEX_UPDATE, update);
		}
	}

	private void deleteAnyChild(LinkedListTree tree) {
		if (tree.getChildCount() > 0) {
			tree.deleteChild(0);
		}
	}
}