/**
 * ASTASIfStatement.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.dom.ASBlock;
import uk.co.badgersinfoil.metaas.dom.ASIfStatement;
import uk.co.badgersinfoil.metaas.dom.StatementContainer;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


public class ASTASIfStatement extends ContainerDelegate implements ASIfStatement {

	private LinkedListTree ast;

	public ASTASIfStatement(LinkedListTree ast) {
		ASTUtils.assertAS3TokenTypeIs(AS3Parser.IF, ast.getType());
		this.ast = ast;
	}

	protected StatementContainer getStatementContainer() {
		return new ASTStatementList((LinkedListTree)ast.getChild(1));
	}

	public ASBlock getElse() {
		LinkedListTree elseStmt = (LinkedListTree)ast.getChild(2);
		if (elseStmt == null) {
			String indent = ASTUtils.findIndent(ast);
			elseStmt = ASTUtils.newAST(AS3Parser.ELSE, "else");
			ast.appendToken(TokenBuilder.newSpace());
			ast.addChildWithTokens(elseStmt);
			elseStmt.appendToken(TokenBuilder.newSpace());
			LinkedListTree block = ASTBuilder.newBlock();
			elseStmt.addChildWithTokens(block);
			ASTUtils.increaseIndentAfterFirstLine(block, indent);
		}
		return new ASTStatementList(elseStmt.getFirstChild());
	}

	public void setThen(ASBlock block) {
		LinkedListTree theBlock = ((ASTStatementList)block).getAST();
		ASTIterator i = new ASTIterator(ast);
		i.next();  // move to condition
		i.next();  // move to then-branch
		i.replace(theBlock);
		String indent = ASTUtils.findIndent(ast);
		ASTUtils.increaseIndentAfterFirstLine(theBlock, indent);
	}

	public String getConditionString() {
		return ASTUtils.stringifyNode(ast.getFirstChild().getFirstChild());
	}

	public void setCondition(String expr) {
		LinkedListTree cond = AS3FragmentParser.parseCondition(expr);
		ast.setChildWithTokens(0, cond);
	}
}