/**
 * ASTASMember.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import java.util.List;

import org.asdt.core.internal.antlr.AS3Parser;

import uk.co.badgersinfoil.metaas.dom.ASMember;
import uk.co.badgersinfoil.metaas.dom.ASMetaTag;
import uk.co.badgersinfoil.metaas.dom.Visibility;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


public abstract class ASTASMember implements ASMember {
	protected LinkedListTree ast;

	public ASTASMember(LinkedListTree ast) {
		this.ast = ast;
	}
	
	public LinkedListTree getAST() {
		return ast;
	}

	public Visibility getVisibility() {
		return ModifierUtils.getVisibility(findModifiers());
	}

	public void setVisibility(Visibility protection) {
		ModifierUtils.setVisibility(findModifiers(), protection);
	}

	public boolean isStatic() {
		LinkedListTree modifiers = findModifiers();
		return ASTUtils.findChildByType(modifiers, AS3Parser.STATIC) != null;
	}

	private LinkedListTree findModifiers() {
		return ASTUtils.findChildByType(ast, AS3Parser.MODIFIERS);
	}

	private ASTIterator modifierIter() {
		return new ASTIterator(findModifiers());
	}

	public void setStatic(boolean s) {
		for (ASTIterator i=modifierIter(); i.hasNext(); ) {
			LinkedListTree mod = i.next();
			if (mod.getType() == AS3Parser.STATIC) {
				if (!s) {
					i.remove();
				}
				return;
			}
		}
		if (s) {
			LinkedListTree modifiers = findModifiers();
			LinkedListTree modStatic = ASTUtils.newAST(AS3Parser.STATIC, "static");
			modStatic.appendToken(TokenBuilder.newSpace());
			modifiers.addChildWithTokens(modStatic);
		}
	}

	public List getAllMetaTags() {
		return TagUtils.getAllMetaTags(ast);
	}

	public ASMetaTag getFirstMetatag(String name) {
		return TagUtils.getFirstMetaTag(ast, name);
	}

	public List getMetaTagsWithName(String name) {
		return TagUtils.getMetaTagWithName(ast, name);
	}

	public ASMetaTag newMetaTag(String name) {
		return TagUtils.newMetaTag(ast, name);
	}
}
