/**
 * ASTForInCommon.java
 * 
 * Copyright (c) 2007 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import uk.co.badgersinfoil.metaas.dom.StatementContainer;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


/**
 * Common code implementing 'for-in' and 'for-each-in' loop behavior.
 */
abstract class ASTForInCommon extends ContainerDelegate {

	private static final int INDEX_VAR = 0;
	private static final int INDEX_ITERATED = 1;
	private static final int INDEX_STATEMENT = 2;
	private LinkedListTree ast;

	public ASTForInCommon(LinkedListTree ast) {
		this.ast = ast;
	}

	public String getVarString() {
		return ASTUtils.stringifyNode(getChild(INDEX_VAR));
	}

	public String getIteratedString() {
		return ASTUtils.stringifyNode(getChild(INDEX_ITERATED));
	}

	public void setVar(String expr) {
		LinkedListTree var = AS3FragmentParser.parseForInVar(expr);
		ast.setChildWithTokens(INDEX_VAR, var);
	}

	public void setIterated(String expr) {
		LinkedListTree iterated = AS3FragmentParser.parseForInIterated(expr);
		ast.setChildWithTokens(INDEX_ITERATED, iterated);
	}

	private LinkedListTree getChild(int index) {
		return (LinkedListTree)ast.getChild(index);
	}

	protected StatementContainer getStatementContainer() {
		return new ASTStatementList(getChild(INDEX_STATEMENT));
	}
}