/**
 * ModifierUtils.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.dom.Visibility;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


/**
 * Helpers for dealing with the modifiers list.
 */
class ModifierUtils {

	public static Visibility getVisibility(LinkedListTree modifiers) {
		for (ASTIterator i=new ASTIterator(modifiers); i.hasNext(); ) {
			LinkedListTree mod = i.next();
			if (mod.getType() == AS3Parser.PRIVATE) {
				return Visibility.PRIVATE;
			}
			if (mod.getType() == AS3Parser.PUBLIC) {
				return Visibility.PUBLIC;
			}
		}
		return Visibility.DEFAULT;
	}

	public static void setVisibility(LinkedListTree modifiers, Visibility protection) {
		for (ASTIterator i=new ASTIterator(modifiers); i.hasNext(); ) {
			LinkedListTree mod = i.next();
			if (mod.getType() == AS3Parser.PRIVATE
			  ||mod.getType() == AS3Parser.PUBLIC)
			{
				if (Visibility.PRIVATE == protection) {
					mod.token.setType(AS3Parser.PRIVATE);
					mod.token.setText("private");
				} else if (Visibility.PUBLIC == protection) {
					mod.token.setType(AS3Parser.PUBLIC);
					mod.token.setText("public");
				} else if (Visibility.DEFAULT == protection) {
					i.remove();
					if (modifiers.getChildCount() == 0) {
						deleteAllChildTokens(modifiers);
					}
				}
				return;
			}
		}
		if (Visibility.PRIVATE == protection) {
			LinkedListTree node = ASTUtils.newAST(AS3Parser.PRIVATE, "private");
			node.appendToken(TokenBuilder.newSpace());
			modifiers.addChildWithTokens(node);
		} else if (Visibility.PUBLIC == protection) {
			LinkedListTree node = ASTUtils.newAST(AS3Parser.PUBLIC, "public");
			node.appendToken(TokenBuilder.newSpace());
			modifiers.addChildWithTokens(node);
		}
	}

	private static void deleteAllChildTokens(LinkedListTree modifiers) {
		for (LinkedListToken tok=modifiers.getStartToken(); tok!=null && tok!=modifiers.getStopToken(); ) {
			LinkedListToken next = tok.getNext();
			tok.delete();
			tok = next;
		}
		modifiers.setStartToken(null);
		modifiers.setStopToken(null);
	}

	/**
	 * Constructs a new MODIFIERS node which represents the given
	 * visibility as an AST containing either "public", "private" or
	 * nothing.
	 */
	public static LinkedListTree toModifiers(Visibility visibility) {
		LinkedListTree modifiers = ASTUtils.newAST(AS3Parser.MODIFIERS);
		if (Visibility.PUBLIC.equals(visibility)) {
			LinkedListTree modPublic = ASTUtils.newAST(AS3Parser.PUBLIC, "public");
			modPublic.appendToken(TokenBuilder.newSpace());
			modifiers.addChildWithTokens(modPublic);
		} else if (Visibility.PRIVATE.equals(visibility)) {
			LinkedListTree modPrivate = ASTUtils.newAST(AS3Parser.PRIVATE, "private");
			modPrivate.appendToken(TokenBuilder.newSpace());
			modifiers.addChildWithTokens(modPrivate);
		}
		return modifiers;
	}
}