/**
 * TokenBuilder.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;


public class TokenBuilder {

	private TokenBuilder() {
		// hide default ctor
	}

	public static LinkedListToken newToken(int type, String text) {
		return new LinkedListToken(type, text);
	}

	public static LinkedListToken newNewline() {
		LinkedListToken nl = newToken(AS3Parser.NL, "\n");
		nl.setChannel(AS3Parser.HIDDEN);
		return nl;
	}

	public static LinkedListToken newSpace() {
		return newWhiteSpace(" ");
	}

	public static LinkedListToken newWhiteSpace(String indentStr) {
		LinkedListToken ws = newToken(AS3Parser.WS, indentStr);
		ws.setChannel(AS3Parser.HIDDEN);
		return ws;
	}

	public static LinkedListToken newClass() {
		return newToken(AS3Parser.CLASS, "class");
	}

	public static LinkedListToken newVar() {
		return newToken(AS3Parser.VAR, "var");
	}

	public static LinkedListToken newComma() {
		return newToken(AS3Parser.COMMA, ",");
	}

	public static LinkedListToken newColon() {
		return newToken(AS3Parser.COLON, ":");
	}

	public static LinkedListToken newSLComment(String text) {
		return newToken(AS3Parser.SL_COMMENT, text);
	}

	public static LinkedListToken newLParen() {
		return newToken(AS3Parser.LPAREN, "(");
	}

	public static LinkedListToken newSemi() {
		return newToken(AS3Parser.SEMI, ";");
	}

	public static LinkedListToken newRParen() {
		return newToken(AS3Parser.RPAREN, ")");
	}

	public static LinkedListToken newIn() {
		return newToken(AS3Parser.IN, "in");
	}

	public static LinkedListToken newWhile() {
		return newToken(AS3Parser.WHILE, "while");
	}

	public static LinkedListToken newMLComment(String text) {
		return newToken(AS3Parser.ML_COMMENT, text);
	}

	public static LinkedListToken newEach() {
		return newToken(AS3Parser.EACH, "each");
	}

	public static LinkedListToken newConst() {
		return newToken(AS3Parser.CONST, "const");
	}

	public static LinkedListToken newInterface() {
		return newToken(AS3Parser.INTERFACE, "interface");
	}
	
	public static LinkedListToken newPlaceholder() {
		LinkedListToken placeholder = new LinkedListToken(AS3Parser.VIRTUAL_PLACEHOLDER, "");
		placeholder.setChannel(AS3Parser.CHANNEL_PLACEHOLDER);
		return placeholder;
	}
}