/**
 * RegexSimpleHelper.java
 * 
 * Copyright (c) 2006-2007 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl.parser;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.CharStream;
import org.antlr.runtime.RecognitionException;
import org.asdt.core.internal.antlr.AS3Lexer;
import uk.co.badgersinfoil.metaas.impl.AS3FragmentParser;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTokenSource;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTokenStream;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTreeAdaptor;
import uk.co.badgersinfoil.metaas.impl.parser.regexsimple.RegexSimpleParser;
import uk.co.badgersinfoil.metaas.impl.parser.regexsimple.RegexSimpleLexer;

public class RegexSimpleHelper {
	private static final LinkedListTreeAdaptor TREE_ADAPTOR = new LinkedListTreeAdaptor();

	/**
	 * Creates a properly-configured parser object for the E4X grammar.
	 */
	public static RegexSimpleParser parserOn(Reader in) throws IOException {
		ANTLRReaderStream cs = new ANTLRReaderStream(in);
		RegexSimpleLexer lexer = new RegexSimpleLexer(cs);
		LinkedListTokenSource source = new LinkedListTokenSource(lexer);
		LinkedListTokenStream stream = new LinkedListTokenStream(source);
		RegexSimpleParser parser = new RegexSimpleParser(stream);
		parser.setTreeAdaptor(TREE_ADAPTOR);
		parser.setInput(lexer, cs);
		return parser;
	}

	public static LinkedListTree parseRegexpLiteral(LinkedListToken startMarker, AS3Lexer lexer, CharStream cs, LinkedListTokenStream stream) throws RecognitionException {
		String tail = startMarker.getText()  // the '/'
		            + cs.substring(cs.index(), cs.size()-1);  // the rest
		int initialTailLength = tail.length();
		RegexSimpleParser parser;
		try {
			parser = regexpParserOn(new StringReader(tail), stream);
		} catch (IOException e) {
			// TODO: better exception type?
			throw new RuntimeException(e);
		}
		LinkedListTree ast = AS3FragmentParser.tree(parser.regexpLiteral());
		tail = parser.getInputTail();
		// skip over the regexp in the original, underlying CharStream
		cs.seek(cs.index() + (initialTailLength - tail.length()) - 1);
		LinkedListTokenSource source = (LinkedListTokenSource)stream.getTokenSource();
		stream.setTokenSource(source);  // cause any remembered RegexpSimple state to be dropped
		stream.scrub(1); // erase the subsequent token that the RegexpSimple parser got from this stream
		source.setDelegate(lexer);
		return ast;
	}

	private static RegexSimpleParser regexpParserOn(Reader in, LinkedListTokenStream stream) throws IOException {
		ANTLRReaderStream cs = new ANTLRReaderStream(in);
		RegexSimpleLexer lexer = new RegexSimpleLexer(cs);
		LinkedListTokenSource source = (LinkedListTokenSource)stream.getTokenSource();
		stream.scrub(1); // erase the '<' and subsequent token that the AS3 parser got from this stream
		source.setDelegate(lexer);
		RegexSimpleParser parser = new RegexSimpleParser(stream);
		parser.setTreeAdaptor(new LinkedListTreeAdaptor());
		parser.setInput(lexer, cs);
		return parser;
	}
}